<?php
if (!defined('ABSPATH')) {
    exit;
}

$ws = new Mbe_Shipping_Model_Ws();
$availableShipping = $ws->getAllowedShipmentServices();

if ($ws->isCustomerActive()) {
    if ($availableShipping) {
        foreach ($availableShipping as $key => $array) {
            $serviceOptions[$array['value']] = $array['label'];
        }
    }
    else {
        $serviceOptions = array('' => __('Set ws parameters and save to retrieve available shipment types', 'wf-shipping-mbe'));
    }
}
else {
    $serviceOptions = array('' => __('Your user is not active', 'wf-shipping-mbe'));
}


$shipmentTypeOptions = array('GENERIC' => __('Generic', 'wf-shipping-mbe'));
$customer = $ws->getCustomer();
if ($customer && $customer->Permissions->canChooseMBEShipType) {
    $shipmentTypeOptions['ENVELOPE'] = __('Envelop', 'wf-shipping-mbe');
}

$countries_obj = new WC_Countries();
$countries = version_compare(WC()->version, '2.1', '>=') ? $countries_obj->__get('countries') : $countries_obj->get_allowed_countries();

// Closure mode options
$closureModeOptions = array(
    Mbe_Shipping_Helper_Data::MBE_CLOSURE_MODE_AUTOMATICALLY => __('Automatically', 'wf-shipping-mbe'),
    Mbe_Shipping_Helper_Data::MBE_CLOSURE_MODE_MANUALLY      => __('Manually', 'wf-shipping-mbe'),
);

$closureTimeOptions = array();
for ($i = 0; $i < 24; $i++) {
    $closureTimeOptions[$i . ':00:00'] = $i . ':00';
    $closureTimeOptions[$i . ':30:00'] = $i . ':30';
}

// Closure mode options
$creationModeOptions = array(
    Mbe_Shipping_Helper_Data::MBE_CREATION_MODE_AUTOMATICALLY => __('Automatically', 'wf-shipping-mbe'),
    Mbe_Shipping_Helper_Data::MBE_CREATION_MODE_MANUALLY      => __('Manually', 'wf-shipping-mbe'),
);


$customInputs = array();
$helper = new Mbe_Shipping_Helper_Data();
$selectedServices = $helper->getAllowedShipmentServices();

if (!empty($selectedServices)) {
    foreach ($selectedServices as $t) {
        $index = array_search($t, array_column($availableShipping, 'value'));
        $shippingLabel = "";
        if (isset($availableShipping[$index]['label'])) {
            $shippingLabel = $availableShipping[$index]['label'];
        }
        $label = sprintf(__('Free shipping Thresholds %s', 'wf-shipping-mbe'), $shippingLabel);
        $customInputs[] = array('label' => $label, 'name' => 'mbelimit_' . strtolower($t));
    }
}

/**
 * Array of settings
 */
$result = array(
    'mbe_active'                   => array(
        'title'   => __('Enabled', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(1 => __('Yes', 'wf-shipping-mbe'), 0 => __('No', 'wf-shipping-mbe')),
        'label'   => __('Enable', 'wf-shipping-mbe'),
        'default' => 'no',
    ),
    'country'                      => array(
        'title'       => __('Country', 'wf-shipping-mbe'),
        'type'        => 'select',
        'options'     => array('IT' => __('Italy', 'wf-shipping-mbe'), 'ES' => __('Spain', 'wf-shipping-mbe'), 'DE' => __('Germany', 'wf-shipping-mbe'), 'FR' => __('France', 'wf-shipping-mbe'), 'AT' => __('Austria', 'wf-shipping-mbe')),
        'description' => __('Country system', 'wf-shipping-mbe'),
        'desc_tip'    => true,
    ),
    'url'                          => array(
        'title'       => __('Web service url', 'wf-shipping-mbe'),
        'type'        => 'text',
        'description' => __('Description web service url', 'wf-shipping-mbe'),
    ),
    'username'                     => array(
        'title' => __('Username', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'password'                     => array(
        'title' => __('Password', 'wf-shipping-mbe'),
        'type'  => 'password',
    ),
    'description'                  => array(
        'title' => __('Description', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'default_shipment_type'        => array(
        'title'   => __('Default Shipment type', 'wf-shipping-mbe'),
        'label'   => __('Default Shipment type', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => $shipmentTypeOptions,
    ),
    'allowed_shipment_services'    => array(
        'title'       => __('MBE Services', 'wf-shipping-mbe'),
        'label'       => __('MBE Services', 'wf-shipping-mbe'),
        'type'        => 'multiselect',
        'options'     => $serviceOptions,
        'description' => __('List of MBE Services enabled for shipping', 'wf-shipping-mbe'),
    ),
    'default_shipment_mode'        => array(
        'title'   => __('Shipment configuration mode', 'wf-shipping-mbe'),
        'label'   => __('Shipment configuration mode', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            '1' => __('Create one Shipment per Item', 'wf-shipping-mbe'),
            '2' => __('Create one Shipment per shopping cart (parcels calculated based on weight)', 'wf-shipping-mbe'),
            '3' => __('Create one Shipment per shopping cart with one parcel per Item', 'wf-shipping-mbe'),
        ),
    ),
    'default_length'               => array(
        'title' => __('Default Package Length', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'default_width'                => array(
        'title' => __('Default Package Width', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'default_height'               => array(
        'title' => __('Default Package Height', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'max_package_weight'           => array(
        'title'       => __('Maximum Package Weight', 'wf-shipping-mbe'),
        'type'        => 'text',
        'description' => __('Please consult your shipping carrier for maximum supported shipping weight', 'wf-shipping-mbe'),
    ),
    'max_shipment_weight'          => array(
        'title' => __('Maximum shipment weight', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'handling_type'                => array(
        'title'   => __('Calculate Handling Fee', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            'P' => __('Percent', 'wf-shipping-mbe'),
            'F' => __('Fixed', 'wf-shipping-mbe'),
        ),
    ),
    'handling_action'              => array(
        'title'   => __('Handling Applied', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            'S' => __('Shipment', 'wf-shipping-mbe'),
            'P' => __('Parcel', 'wf-shipping-mbe'),
        ),
    ),
    'handling_fee'                 => array(
        'title' => __('Handling Fee', 'wf-shipping-mbe'),
        'type'  => 'text',
    ),
    'handling_fee_rounding'        => array(
        'title'   => __('Handling fee calculation: apply rounding', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            '1' => __('Nessun arrotondamento', 'wf-shipping-mbe'),
            '2' => __('Applica arrotondamento di default', 'wf-shipping-mbe'),
            '3' => __('Arrotondamento per difetto', 'wf-shipping-mbe'),
            '4' => __('Arrotondamento per eccesso', 'wf-shipping-mbe'),
        ),
    ),
    'handling_fee_rounding_amount' => array(
        'title'   => __('Handling fee calculation: rounding amount', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            '1' => '1',
            '2' => '0.5',
        ),
    ),
    'sallowspecific'               => array(
        'title'   => __('Ship to Applicable Countries', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => array(
            '0' => __('All Allowed Countries', 'wf-shipping-mbe'),
            '1' => __('Specific Countries', 'wf-shipping-mbe'),
        ),
    ),
    'specificcountry'              => array(
        'title'   => __('Country', 'wf-shipping-mbe'),
        'type'    => 'multiselect',
        'options' => $countries,
    ),
//	'sort_order' => array(
//		'title'           => __( 'Sort Order', 'wf-shipping-mbe' ),
//		'type'            => 'text',
//	),
//	'mbe_shipping_specificerrmsg' => array(
//		'title'           => __( 'Displayed Error Message', 'wf-shipping-mbe' ),
//		'type'            => 'textarea',
//	),
    'shipments_closure_mode'       => array(
        'title'   => __('Shipments closure mode', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => $closureModeOptions,
    ),
    'shipments_closure_time'       => array(
        'title'   => __('Shipments closure time', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => $closureTimeOptions,
    ),
    'shipments_creation_mode'      => array(
        'title'   => __('Shipments creation mode', 'wf-shipping-mbe'),
        'type'    => 'select',
        'options' => $creationModeOptions,
    ),
);

foreach ($customInputs as $input) {
    $result[$input['name']] = array(
        'title' => __($input['label'], 'wf-shipping-mbe'),
        'type'  => 'text',
    );
}

return $result;